@@ -4,6 +4,7 @@ import csv
 import importlib.resources
 import json
 import logging
+import os
 import platform
 import random
 import ssl
@@ -30,6 +31,7 @@ from hydroqc.hydro_api.consts import (
     CONTRACT_SUMMARY_URL,
     CUSTOMER_INFO_URL,
     DAILY_CONSUMPTION_API_URL,
+    FLEXD_DATA_URL,
     GET_WINTER_CREDIT_API_URL,
     HOURLY_CONSUMPTION_API_URL,
     LOGIN_URL_6,
@@ -52,6 +54,7 @@ from hydroqc.types import (
     ConsoHourlyTyping,
     ConsoMonthlyTyping,
     ContractSummaryTyping,
+    DPCDataTyping,
     IDTokenTyping,
     InfoCompteTyping,
     ListContractsTyping,
@@ -79,6 +82,7 @@ class HydroClient:
         verify_ssl: bool = True,
         session: aiohttp.ClientSession | None = None,
         log_level: str | None = "INFO",
+        diag_folder: str | None = None,
     ):
         """Initialize the client object."""
         self.username: str = username
@@ -86,6 +90,8 @@ class HydroClient:
         self._timeout: int = timeout
         self._session: aiohttp.ClientSession | None = session
         self._verify_ssl: bool = verify_ssl
+        self._diag_folder: str | None = diag_folder
+        self._diag_id: int = 0
         with importlib.resources.path(
             __package__, "hydro-chain.pem"
         ) as hydro_chain_pem_file:
@@ -149,6 +155,26 @@ class HydroClient:
         if verify_ssl:
             ssl_context = self._sslcontext
 
+        # Query Diagnostic
+        if self._diag_folder is not None:
+            cleaned_url = url.replace("/", "_").split("#", 1)[0]
+            diag_file_path = os.path.join(
+                self._diag_folder,
+                f"hq_{self._diag_id:02}_q_{method}_{cleaned_url}.json"[:128],
+            )
+            with open(diag_file_path, "w", encoding="utf-8") as fhj:
+                json.dump(
+                    {
+                        "url": url,
+                        "params": params,
+                        "data": data,
+                        # TODO Remove tokens
+                        "headers": headers,
+                    },
+                    fhj,
+                    indent=2,
+                )
+
         self._logger.debug("HTTP query %s to %s", url, method)
         raw_res: aiohttp.ClientResponse = await getattr(self._session, method)(
             url,
@@ -159,6 +185,25 @@ class HydroClient:
             cookies=cookies,
             headers=headers,
         )
+        # Result Diagnostic
+        if self._diag_folder is not None:
+            cleaned_url = url.replace("/", "_").split("#", 1)[0]
+            diag_file_path = os.path.join(
+                self._diag_folder,
+                f"hq_{self._diag_id:02}_r_{method}_{cleaned_url}.json"[:128],
+            )
+            diag_result = await raw_res.text()
+            try:
+                diag_result = json.dumps(json.loads(diag_result), indent=2)
+
+            except json.JSONDecodeError:
+                pass
+
+            with open(diag_file_path, "w", encoding="utf-8") as fhj:
+                fhj.write(diag_result)
+            # Increment diag_id
+            self._diag_id += 1
+
         if raw_res.status != status:
             self._logger.exception("Exception in http_request")
             data = await raw_res.text()
@@ -458,6 +503,7 @@ class HydroClient:
             raise HydroQcError("Account not found")
         return accounts[account_id]
 
+    @CCached(ttl=21600)
     async def list_account_contract(
         self, applicant_id: str, customer_id: str
     ) -> ContractSummaryTyping:
@@ -647,7 +693,23 @@ class HydroClient:
         data: ConsoAnnualTyping = self._load_json(await api_call_response.text())
         return data
 
-    async def get_conso_csv(
+    @CCached(ttl=21600)
+    async def get_dpc_data(
+        self, applicant_id: str, customer_id: str, contract_id: str
+    ) -> DPCDataTyping:
+        """Fetch FlexD data of the current year.
+
+        API URL: https://cl-ec-spring.hydroquebec.com/portail/fr/group/clientele/
+        portrait-de-consommation/resourceObtenirDonneesMoisHiverFlex
+
+        """
+        await self._select_contract(applicant_id, customer_id, contract_id)
+        api_call_response = await self.http_request(FLEXD_DATA_URL, "get")
+        # We can not use res.json() because the response header are not application/json
+        data: DPCDataTyping = self._load_json(await api_call_response.text())
+        return data
+
+    async def get_consumption_csv(
         self,
         applicant_id: str,
         customer_id: str,
@@ -677,7 +739,7 @@ class HydroClient:
         data_csv: Iterator[Any] = csv.reader(content, delimiter=";")
         return data_csv
 
-    async def get_conso_overview_csv(
+    async def get_consumption_overview_csv(
         self,
         applicant_id: str,
         customer_id: str,
